# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>


from .parser_manager import ParserManager


def parse(sip_file, hex_version, encoding, target_abi, tags,
        disabled_features, protected_is_public, include_dirs, sip_module,
        is_strict=True):
    """ Parse a .sip file and return a 3-tuple of a Specification object, a
    list of Module objects and a list of the .sip files that specify the module
    to be generated.  A UserException is raised if there was an error.
    """

    return ParserManager(
            hex_version, encoding, target_abi, tags, disabled_features,
            protected_is_public, include_dirs, sip_module, is_strict).parse(
                    sip_file)



# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


from ...scoped_name import STRIP_NONE
from ...specification import ArgumentType

from .argument import (fmt_argument_as_cpp_type, fmt_argument_as_name,
        fmt_argument_as_type_hint)


def fmt_signature_as_cpp_declaration(spec, cpp_signature, scope=None,
        strip=STRIP_NONE, make_public=False, as_xml=False):
    """ Return the C++ representation of a signature's arguments as a
    declaration.
    """

    args = [fmt_argument_as_cpp_type(spec, arg, scope=scope, strip=strip,
                    make_public=make_public, as_xml=as_xml)
                for arg in cpp_signature.args]

    return ', '.join(args)


def fmt_signature_as_cpp_definition(spec, cpp_signature, scope=None,
        make_public=False):
    """ Return the C++ representation of a signature's arguments as a
    definition.
    """

    args = []

    for arg_nr, arg in enumerate(cpp_signature.args):
        arg_name = fmt_argument_as_name(spec, arg, arg_nr)

        args.append(
                fmt_argument_as_cpp_type(spec, arg, name=arg_name, scope=scope,
                        make_public=make_public))

    return ', '.join(args)


def fmt_signature_as_type_hint(spec, py_signature, need_self=True,
        exclude_result=False, defined=None):
    """ Return a signature as a type hint. """

    # Handle the input values.
    in_args = []

    if need_self:
        in_args.append('self')

    nr_out = 0

    for arg_nr, arg in enumerate(py_signature.args):
        if arg.is_out:
            nr_out += 1

        if arg.is_in:
            type_hint = fmt_argument_as_type_hint(spec, arg, defined,
                    arg_nr=arg_nr)
            if type_hint:
                in_args.append(type_hint)

    args_s = '(' + ', '.join(in_args) +')'

    if exclude_result:
        return args_s

    # Handle the output values.
    result = py_signature.result

    if result is None or (result.type is ArgumentType.VOID and len(result.derefs) == 0):
        is_result = False
    else:
        type_hints = result.type_hints

        # An empty type hint specifies a void return.
        if type_hints is not None and type_hints.hint_out is not None and type_hints.hint_out == '':
            is_result = False
        else:
            is_result = True

    if is_result or nr_out > 0:
        results_s = ' -> '

        needs_tuple = ((is_result and nr_out > 0) or nr_out > 1)

        if needs_tuple:
            results_s += '(' if defined is None else 'typing.Tuple['

        out_args = []

        if is_result:
            type_hint = fmt_argument_as_type_hint(spec, result, defined)
            if type_hint:
                out_args.append(type_hint)

        for arg in py_signature.args:
            if arg.is_out:
                type_hint = fmt_argument_as_type_hint(spec, arg, defined)
                if type_hint:
                    out_args.append(type_hint)

        results_s += ', '.join(out_args)

        if needs_tuple:
            results_s += ')' if defined is None else ']'
    elif defined is not None:
        results_s = ' -> None'
    else:
        # We don't show 'None' returns in docstrings.
        results_s = ''

    return f'{args_s}{results_s}'



# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>


# The major number of the default target ABI.  Remember to update the
# documentation.
DEFAULT_ABI_MAJOR = 13

# The minimum required version of setuptools.  This is the earliest version
# that generates correct wheel names for PyPI.  Remember to update
# pyproject.toml in the root directory.
MINIMUM_SETUPTOOLS = '75.8.1'

# The oldest supported minor version of Python v3.  Remember to update
# pyproject.toml in the root directory.
OLDEST_SUPPORTED_MINOR = 10



# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


from .abstract_project import AbstractProject
from .exceptions import handle_exception


def build_sdist(sdist_directory, config_settings=None):
    """ The PEP 517 hook for building an sdist from pyproject.toml. """

    project = AbstractProject.bootstrap('sdist',
            arguments=_convert_config_settings(config_settings))

    # pip executes this in a separate process and doesn't handle exceptions
    # very well.  However it does capture stdout and (eventually) show it to
    # the user so we use our standard exception handling.
    try:
        return project.build_sdist(sdist_directory)
    except Exception as e:
        handle_exception(e)


def build_wheel(wheel_directory, config_settings=None, metadata_directory=None):
    """ The PEP 517 hook for building a wheel from pyproject.toml. """

    project = AbstractProject.bootstrap('wheel',
            arguments=_convert_config_settings(config_settings))

    # pip executes this in a separate process and doesn't handle exceptions
    # very well.  However it does capture stdout and (eventually) show it to
    # the user so we use our standard exception handling.
    try:
        return project.build_wheel(wheel_directory)
    except Exception as e:
        handle_exception(e)


def _convert_config_settings(config_settings):
    """ Return any configuration settings from the frontend to a pseudo-command
    line.
    """

    if config_settings is None:
        config_settings = {}

    args = []

    for name, value in config_settings.items():
        if value:
            if not isinstance(value, list):
                value = [value]

            for m_value in value:
                args.append(name + '=' + m_value)
        else:
            args.append(name)

    return args
